"""
MIT License

Copyright (c) 2019 Andrew Riha

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.

"""

import os
import shutil
from unittest import TestCase

import numpy as np
import pandas as pd
from snps.io.reader import NORMALIZED_DTYPES

from lineage import Lineage


class BaseLineageTestCase(TestCase):
    def setUp(self):
        self.l = Lineage()
        self.del_output_dir_helper()

    def tearDown(self):
        self.del_output_dir_helper()

    @staticmethod
    def del_output_dir_helper():
        if os.path.exists("output"):
            shutil.rmtree("output")

    def simulate_snps(
        self,
        ind,
        chrom="1",
        pos_start=1,
        pos_max=111700002,
        pos_step=10000,
        genotype="AA",
        insert_nulls=True,
        null_snp_step=101,
        complement_genotype_one_chrom=False,
        complement_genotype_two_chroms=False,
        complement_snp_step=50,
    ):
        ind._build = 37

        positions = np.arange(pos_start, pos_max, pos_step, dtype=np.int64)
        snps = pd.DataFrame(
            {"chrom": chrom},
            index=pd.Index(
                ["rs" + str(x + 1) for x in range(len(positions))], name="rsid"
            ),
        )
        snps["pos"] = positions
        snps["genotype"] = genotype

        if insert_nulls:
            snps.loc[snps.iloc[0::null_snp_step, :].index, "genotype"] = np.nan

        indices = snps.iloc[0::complement_snp_step, :].index
        if complement_genotype_two_chroms:
            snps.loc[indices, "genotype"] = snps.loc[indices, "genotype"].apply(
                self.complement_two_chroms
            )
        elif complement_genotype_one_chrom:
            snps.loc[indices, "genotype"] = snps.loc[indices, "genotype"].apply(
                self.complement_one_chrom
            )

        ind._snps = snps

        return ind

    @staticmethod
    def get_complement(base):
        if base == "A":
            return "T"
        elif base == "G":
            return "C"
        elif base == "C":
            return "G"
        elif base == "T":
            return "A"
        else:
            return base

    def complement_one_chrom(self, genotype):
        if pd.isnull(genotype):
            return np.nan

        complement = ""

        for base in list(genotype):
            complement += self.get_complement(base)
            complement += genotype[1]
            return complement

    def complement_two_chroms(self, genotype):
        if pd.isnull(genotype):
            return np.nan

        complement = ""

        for base in list(genotype):
            complement += self.get_complement(base)

        return complement

    @staticmethod
    def create_snp_df(rsid, chrom, pos, genotype):
        df = pd.DataFrame(
            {"rsid": rsid, "chrom": chrom, "pos": pos, "genotype": genotype},
            columns=["rsid", "chrom", "pos", "genotype"],
        )
        df = df.astype(NORMALIZED_DTYPES)
        df = df.set_index("rsid")
        return df

    def generic_snps(self):
        return self.create_snp_df(
            rsid=["rs" + str(i) for i in range(1, 9)],
            chrom=["1"] * 8,
            pos=list(range(101, 109)),
            genotype=["AA", "CC", "GG", "TT", np.nan, "GC", "TC", "AT"],
        )



"""
MIT License

Copyright (c) 2017 Andrew Riha

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.

"""

import os
import warnings

from lineage.resources import Resources
from tests import BaseLineageTestCase


class TestResources(BaseLineageTestCase):
    def setUp(self):
        self.resource = Resources(resources_dir="resources")
        self.del_output_dir_helper()

    def test_get_genetic_map_HapMapII_GRCh37(self):
        genetic_map_HapMapII_GRCh37 = self.resource.get_genetic_map_HapMapII_GRCh37()
        assert len(genetic_map_HapMapII_GRCh37) == 23

    def test_get_cytoBand_hg19(self):
        cytoBand_hg19 = self.resource.get_cytoBand_hg19()
        assert len(cytoBand_hg19) == 862

    def test_get_knownGene_hg19(self):
        knownGene_hg19 = self.resource.get_knownGene_hg19()
        assert len(knownGene_hg19) == 82960

    def test_get_kgXref_hg19(self):
        kgXref_hg19 = self.resource.get_kgXref_hg19()
        assert len(kgXref_hg19) == 82960

    def test_get_all_resources(self):
        resources = self.resource.get_all_resources()
        for k, v in resources.items():
            if v is None:
                assert False
        assert True

    def test_download_example_datasets(self):
        paths = self.resource.download_example_datasets()

        for path in paths:
            if path is None or not os.path.exists(path):
                warnings.warn("Example dataset(s) not currently available")
                return

        assert True



"""
MIT License

Copyright (c) 2018 Andrew Riha

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.

"""

import os
import warnings

import numpy as np
import pandas as pd

from tests import BaseLineageTestCase


class TestLineage(BaseLineageTestCase):
    def get_discordant_snps(self, ind, df):
        ind._build = 37

        snps = df.loc[:, ["chrom", "pos", ind.name]]
        snps = snps.rename(columns={ind.name: "genotype"})

        ind._snps = snps

        return ind

    def test_download_example_datasets(self):
        paths = self.l.download_example_datasets()

        for path in paths:
            if path is None or not os.path.exists(path):
                warnings.warn("Example dataset(s) not currently available")
                return

        assert True

    def test_find_discordant_snps(self):
        df = pd.read_csv(
            "tests/input/discordant_snps.csv",
            skiprows=1,
            na_values="--",
            names=["rsid", "chrom", "pos", "ind1", "ind2", "ind3"],
            index_col=0,
            dtype={"chrom": object, "pos": np.int64},
        )

        ind1 = self.get_discordant_snps(self.l.create_individual("ind1"), df)
        ind2 = self.get_discordant_snps(self.l.create_individual("ind2"), df)
        ind3 = self.get_discordant_snps(self.l.create_individual("ind3"), df)

        df_ind1_ind2 = self.l.find_discordant_snps(ind1, ind2, save_output=True)
        df_ind1_ind2_ind3 = self.l.find_discordant_snps(
            ind1, ind2, ind3, save_output=True
        )

        pd.testing.assert_index_equal(
            pd.Index(
                [
                    "rs34",
                    "rs47",
                    "rs48",
                    "rs49",
                    "rs50",
                    "rs106",
                    "rs110",
                    "rs111",
                    "rs112",
                    "rs113",
                    "rs137",
                    "rs140",
                    "rs141",
                    "rs144",
                    "rs146",
                    "rs147",
                ],
                name="rsid",
            ),
            df_ind1_ind2.index,
        )

        pd.testing.assert_index_equal(
            pd.Index(
                [
                    "rs30",
                    "rs34",
                    "rs38",
                    "rs42",
                    "rs46",
                    "rs47",
                    "rs48",
                    "rs49",
                    "rs50",
                    "rs60",
                    "rs75",
                    "rs85",
                    "rs100",
                    "rs102",
                    "rs106",
                    "rs110",
                    "rs111",
                    "rs112",
                    "rs113",
                    "rs114",
                    "rs118",
                    "rs122",
                    "rs135",
                    "rs137",
                    "rs139",
                    "rs140",
                    "rs141",
                    "rs142",
                    "rs144",
                    "rs146",
                    "rs147",
                    "rs148",
                ],
                name="rsid",
            ),
            df_ind1_ind2_ind3.index,
        )

        assert os.path.exists("output/discordant_snps_ind1_ind2_GRCh37.csv")
        assert os.path.exists("output/discordant_snps_ind1_ind2_ind3_GRCh37.csv")

    def test_find_shared_dna_one_ind(self):
        ind1 = self.simulate_snps(self.l.create_individual("ind1"))

        d = self.l.find_shared_dna([ind1], shared_genes=True)

        assert len(d["one_chrom_shared_dna"]) == 0
        assert len(d["two_chrom_shared_dna"]) == 0
        assert len(d["one_chrom_shared_genes"]) == 0
        assert len(d["two_chrom_shared_genes"]) == 0
        assert len(d["one_chrom_discrepant_snps"]) == 0
        assert len(d["two_chrom_discrepant_snps"]) == 0
        assert not os.path.exists("output/shared_dna_one_chrom_ind1_GRCh37.csv")
        assert not os.path.exists("output/shared_dna_two_chroms_ind1_GRCh37.csv")
        assert not os.path.exists("output/shared_genes_one_chrom_ind1_GRCh37.csv")
        assert not os.path.exists("output/shared_genes_two_chroms_ind1_GRCh37.csv")
        assert not os.path.exists("output/shared_dna_ind1.png")

    def test_find_shared_dna_two_chrom_shared(self):
        ind1 = self.simulate_snps(self.l.create_individual("ind1"))
        ind2 = self.simulate_snps(self.l.create_individual("ind2"))

        d = self.l.find_shared_dna([ind1, ind2], shared_genes=True)

        assert len(d["one_chrom_shared_dna"]) == 1
        assert len(d["two_chrom_shared_dna"]) == 1
        assert len(d["one_chrom_shared_genes"]) == 3811
        assert len(d["two_chrom_shared_genes"]) == 3811
        assert len(d["one_chrom_discrepant_snps"]) == 0
        assert len(d["two_chrom_discrepant_snps"]) == 0
        np.testing.assert_allclose(d["one_chrom_shared_dna"].loc[1]["cMs"], 140.443968)
        np.testing.assert_allclose(d["two_chrom_shared_dna"].loc[1]["cMs"], 140.443968)
        assert os.path.exists("output/shared_dna_one_chrom_ind1_ind2_GRCh37.csv")
        assert os.path.exists("output/shared_dna_two_chroms_ind1_ind2_GRCh37.csv")
        assert os.path.exists("output/shared_genes_one_chrom_ind1_ind2_GRCh37.csv")
        assert os.path.exists("output/shared_genes_two_chroms_ind1_ind2_GRCh37.csv")
        assert os.path.exists("output/shared_dna_ind1_ind2.png")

    def test_find_shared_dna_two_chrom_shared_three_ind(self):
        ind1 = self.simulate_snps(self.l.create_individual("ind1"))
        ind2 = self.simulate_snps(self.l.create_individual("ind2"))
        ind3 = self.simulate_snps(self.l.create_individual("ind3"))

        d = self.l.find_shared_dna([ind1, ind2, ind3], shared_genes=True)

        assert len(d["one_chrom_shared_dna"]) == 1
        assert len(d["two_chrom_shared_dna"]) == 1
        assert len(d["one_chrom_shared_genes"]) == 3811
        assert len(d["two_chrom_shared_genes"]) == 3811
        assert len(d["one_chrom_discrepant_snps"]) == 0
        assert len(d["two_chrom_discrepant_snps"]) == 0
        np.testing.assert_allclose(d["one_chrom_shared_dna"].loc[1]["cMs"], 140.443968)
        np.testing.assert_allclose(d["two_chrom_shared_dna"].loc[1]["cMs"], 140.443968)
        assert os.path.exists("output/shared_dna_one_chrom_ind1_ind2_ind3_GRCh37.csv")
        assert os.path.exists("output/shared_dna_two_chroms_ind1_ind2_ind3_GRCh37.csv")
        assert os.path.exists("output/shared_genes_one_chrom_ind1_ind2_ind3_GRCh37.csv")
        assert os.path.exists(
            "output/shared_genes_two_chroms_ind1_ind2_ind3_GRCh37.csv"
        )
        assert os.path.exists("output/shared_dna_ind1_ind2_ind3.png")

    def test_find_shared_dna_two_chrom_shared_no_output(self):
        ind1 = self.simulate_snps(self.l.create_individual("ind1"))
        ind2 = self.simulate_snps(self.l.create_individual("ind2"))

        d = self.l.find_shared_dna([ind1, ind2], shared_genes=True, save_output=False)

        assert len(d["one_chrom_shared_dna"]) == 1
        assert len(d["two_chrom_shared_dna"]) == 1
        assert len(d["one_chrom_shared_genes"]) == 3811
        assert len(d["two_chrom_shared_genes"]) == 3811
        assert len(d["one_chrom_discrepant_snps"]) == 0
        assert len(d["two_chrom_discrepant_snps"]) == 0
        np.testing.assert_allclose(d["one_chrom_shared_dna"].loc[1]["cMs"], 140.443968)
        np.testing.assert_allclose(d["two_chrom_shared_dna"].loc[1]["cMs"], 140.443968)
        assert not os.path.exists("output/shared_dna_one_chrom_ind1_ind2_GRCh37.csv")
        assert not os.path.exists("output/shared_dna_two_chroms_ind1_ind2_GRCh37.csv")
        assert not os.path.exists("output/shared_genes_one_chrom_ind1_ind2_GRCh37.csv")
        assert not os.path.exists("output/shared_genes_two_chroms_ind1_ind2_GRCh37.csv")
        assert not os.path.exists("output/shared_dna_ind1_ind2.png")

    def test_find_shared_dna_one_chrom_shared(self):
        ind1 = self.simulate_snps(self.l.create_individual("ind1"))
        ind2 = self.simulate_snps(
            self.l.create_individual("ind2"), complement_genotype_one_chrom=True
        )

        d = self.l.find_shared_dna([ind1, ind2], shared_genes=True)

        assert len(d["one_chrom_shared_dna"]) == 1
        assert len(d["two_chrom_shared_dna"]) == 0
        assert len(d["one_chrom_shared_genes"]) == 3811
        assert len(d["two_chrom_shared_genes"]) == 0
        assert len(d["one_chrom_discrepant_snps"]) == 0
        assert len(d["two_chrom_discrepant_snps"]) == 0
        np.testing.assert_allclose(d["one_chrom_shared_dna"].loc[1]["cMs"], 140.443968)
        assert os.path.exists("output/shared_dna_one_chrom_ind1_ind2_GRCh37.csv")
        assert not os.path.exists("output/shared_dna_two_chroms_ind1_ind2_GRCh37.csv")
        assert os.path.exists("output/shared_genes_one_chrom_ind1_ind2_GRCh37.csv")
        assert not os.path.exists("output/shared_genes_two_chroms_ind1_ind2_GRCh37.csv")
        assert os.path.exists("output/shared_dna_ind1_ind2.png")

    def test_find_shared_dna_one_chrom_shared_three_ind(self):
        ind1 = self.simulate_snps(self.l.create_individual("ind1"))
        ind2 = self.simulate_snps(
            self.l.create_individual("ind2"), complement_genotype_one_chrom=True
        )
        ind3 = self.simulate_snps(self.l.create_individual("ind3"))

        d = self.l.find_shared_dna([ind1, ind2, ind3], shared_genes=True)

        assert len(d["one_chrom_shared_dna"]) == 1
        assert len(d["two_chrom_shared_dna"]) == 0
        assert len(d["one_chrom_shared_genes"]) == 3811
        assert len(d["two_chrom_shared_genes"]) == 0
        assert len(d["one_chrom_discrepant_snps"]) == 0
        assert len(d["two_chrom_discrepant_snps"]) == 0
        np.testing.assert_allclose(d["one_chrom_shared_dna"].loc[1]["cMs"], 140.443968)
        assert os.path.exists("output/shared_dna_one_chrom_ind1_ind2_ind3_GRCh37.csv")
        assert not os.path.exists(
            "output/shared_dna_two_chroms_ind1_ind2_ind3_GRCh37.csv"
        )
        assert os.path.exists("output/shared_genes_one_chrom_ind1_ind2_ind3_GRCh37.csv")
        assert not os.path.exists(
            "output/shared_genes_two_chroms_ind1_ind2_ind3_GRCh37.csv"
        )
        assert os.path.exists("output/shared_dna_ind1_ind2_ind3.png")

    def test_find_shared_dna_X_chrom_two_individuals_male(self):
        ind1 = self.simulate_snps(
            self.l.create_individual("ind1"),
            chrom="X",
            pos_max=155270560,
            pos_step=1000,
            genotype="AA",
        )
        ind2 = self.simulate_snps(
            self.l.create_individual("ind2"),
            chrom="X",
            pos_max=155270560,
            pos_step=1000,
            genotype="AA",
        )

        d = self.l.find_shared_dna([ind1, ind2], shared_genes=True)

        assert len(d["one_chrom_shared_dna"]) == 1  # PAR1, non-PAR, PAR2
        assert len(d["two_chrom_shared_dna"]) == 1  # PAR1
        assert len(d["one_chrom_shared_genes"]) == 3022
        assert len(d["two_chrom_shared_genes"]) == 54
        assert len(d["one_chrom_discrepant_snps"]) == 0
        assert len(d["two_chrom_discrepant_snps"]) == 0
        np.testing.assert_allclose(d["one_chrom_shared_dna"].loc[1]["cMs"], 202.022891)
        np.testing.assert_allclose(d["two_chrom_shared_dna"].loc[1]["cMs"], 20.837792)
        assert os.path.exists("output/shared_dna_one_chrom_ind1_ind2_GRCh37.csv")
        assert os.path.exists("output/shared_dna_two_chroms_ind1_ind2_GRCh37.csv")
        assert os.path.exists("output/shared_genes_one_chrom_ind1_ind2_GRCh37.csv")
        assert os.path.exists("output/shared_genes_two_chroms_ind1_ind2_GRCh37.csv")
        assert os.path.exists("output/shared_dna_ind1_ind2.png")

    def test_find_shared_dna_X_chrom_two_individuals_female(self):
        ind1 = self.simulate_snps(
            self.l.create_individual("ind1"),
            chrom="X",
            pos_max=155270560,
            genotype="AC",
        )
        ind2 = self.simulate_snps(
            self.l.create_individual("ind2"),
            chrom="X",
            pos_max=155270560,
            genotype="AC",
        )

        d = self.l.find_shared_dna([ind1, ind2], shared_genes=True)

        assert len(d["one_chrom_shared_dna"]) == 1  # PAR1, non-PAR, PAR2
        assert len(d["two_chrom_shared_dna"]) == 1  # PAR1, non-PAR, PAR2
        assert len(d["one_chrom_shared_genes"]) == 3022
        assert len(d["two_chrom_shared_genes"]) == 3022
        assert len(d["one_chrom_discrepant_snps"]) == 0
        assert len(d["two_chrom_discrepant_snps"]) == 0
        np.testing.assert_allclose(d["one_chrom_shared_dna"].loc[1]["cMs"], 202.022891)
        np.testing.assert_allclose(d["two_chrom_shared_dna"].loc[1]["cMs"], 202.022891)
        assert os.path.exists("output/shared_dna_one_chrom_ind1_ind2_GRCh37.csv")
        assert os.path.exists("output/shared_dna_two_chroms_ind1_ind2_GRCh37.csv")
        assert os.path.exists("output/shared_genes_one_chrom_ind1_ind2_GRCh37.csv")
        assert os.path.exists("output/shared_genes_two_chroms_ind1_ind2_GRCh37.csv")
        assert os.path.exists("output/shared_dna_ind1_ind2.png")

    def test_find_shared_dna_two_chrom_shared_discrepant_snps(self):
        # simulate discrepant SNPs so that stitching of adjacent shared DNA segments is performed
        ind1 = self.simulate_snps(self.l.create_individual("ind1"))
        ind2 = self.simulate_snps(
            self.l.create_individual("ind2"),
            complement_genotype_one_chrom=True,
            complement_snp_step=5000,
        )

        d = self.l.find_shared_dna([ind1, ind2], shared_genes=True)

        assert len(d["one_chrom_shared_dna"]) == 1
        assert len(d["two_chrom_shared_dna"]) == 1
        assert len(d["one_chrom_shared_genes"]) == 3811
        assert len(d["two_chrom_shared_genes"]) == 3811
        assert len(d["one_chrom_discrepant_snps"]) == 0
        assert len(d["two_chrom_discrepant_snps"]) == 2
        np.testing.assert_allclose(d["one_chrom_shared_dna"].loc[1]["cMs"], 140.443968)
        np.testing.assert_allclose(d["two_chrom_shared_dna"].loc[1]["cMs"], 140.443968)
        assert os.path.exists("output/shared_dna_one_chrom_ind1_ind2_GRCh37.csv")
        assert os.path.exists("output/shared_dna_two_chroms_ind1_ind2_GRCh37.csv")
        assert os.path.exists("output/shared_genes_one_chrom_ind1_ind2_GRCh37.csv")
        assert os.path.exists("output/shared_genes_two_chroms_ind1_ind2_GRCh37.csv")
        assert os.path.exists("output/shared_dna_ind1_ind2.png")

    def test_find_shared_dna_no_shared_dna(self):
        ind1 = self.simulate_snps(self.l.create_individual("ind1"))
        ind2 = self.simulate_snps(
            self.l.create_individual("ind2"), complement_genotype_two_chroms=True
        )

        d = self.l.find_shared_dna([ind1, ind2], shared_genes=True)

        assert len(d["one_chrom_shared_dna"]) == 0
        assert len(d["two_chrom_shared_dna"]) == 0
        assert len(d["one_chrom_shared_genes"]) == 0
        assert len(d["two_chrom_shared_genes"]) == 0
        assert len(d["one_chrom_discrepant_snps"]) == 0
        assert len(d["two_chrom_discrepant_snps"]) == 0
        assert not os.path.exists("output/shared_dna_one_chrom_ind1_ind2_GRCh37.csv")
        assert not os.path.exists("output/shared_dna_two_chroms_ind1_ind2_GRCh37.csv")
        assert not os.path.exists("output/shared_genes_one_chrom_ind1_ind2_GRCh37.csv")
        assert not os.path.exists("output/shared_genes_two_chroms_ind1_ind2_GRCh37.csv")
        assert os.path.exists("output/shared_dna_ind1_ind2.png")

    def test_find_shared_dna_no_shared_dna_three_ind(self):
        ind1 = self.simulate_snps(self.l.create_individual("ind1"))
        ind2 = self.simulate_snps(
            self.l.create_individual("ind2"), complement_genotype_two_chroms=True
        )
        ind3 = self.simulate_snps(self.l.create_individual("ind3"))

        d = self.l.find_shared_dna([ind1, ind2, ind3], shared_genes=True)

        assert len(d["one_chrom_shared_dna"]) == 0
        assert len(d["two_chrom_shared_dna"]) == 0
        assert len(d["one_chrom_shared_genes"]) == 0
        assert len(d["two_chrom_shared_genes"]) == 0
        assert len(d["one_chrom_discrepant_snps"]) == 0
        assert len(d["two_chrom_discrepant_snps"]) == 0
        assert not os.path.exists(
            "output/shared_dna_one_chrom_ind1_ind2_ind3_GRCh37.csv"
        )
        assert not os.path.exists(
            "output/shared_dna_two_chroms_ind1_ind2_ind3_GRCh37.csv"
        )
        assert not os.path.exists(
            "output/shared_genes_one_chrom_ind1_ind2_ind3_GRCh37.csv"
        )
        assert not os.path.exists(
            "output/shared_genes_two_chroms_ind1_ind2_ind3_GRCh37.csv"
        )
        assert os.path.exists("output/shared_dna_ind1_ind2_ind3.png")
